#include "livemodethread.h"
#include "qhyccd.h"
#include <QDateTime>
#include <cstdio>
#include <QDebug>
#include <QMutex>

extern qhyccd_handle *camhandle;

extern QMutex ShowImgMutex;

LiveModeThread::LiveModeThread(QObject *parent) :
    QThread(parent)
{

}

void LiveModeThread::run()
{
    uint32_t ret = QHYCCD_ERROR;
    CamInfo.Frame = 0;
    CamInfo.FrameLast = 0;
    CamInfo.TimeLast = QDateTime::currentMSecsSinceEpoch();
    while(true)
    {
        while(ret == QHYCCD_ERROR && !CamInfo.Quit)
        {
            ret = GetQHYCCDLiveFrame(camhandle, &CamInfo.ImgW, &CamInfo.ImgH, &CamInfo.ImgB, &CamInfo.ImgC, CamInfo.ImgData);
            if(ret == QHYCCD_SUCCESS)
            {
                CamInfo.Frame ++;
                CamInfo.Time = QDateTime::currentMSecsSinceEpoch();
                emit showFrame();

                if(CamInfo.Time - CamInfo.TimeLast > 500)
                {
                    CamInfo.FrameCount = CamInfo.Frame - CamInfo.FrameLast;
                    CamInfo.FrameLast  = CamInfo.Frame;
                    CamInfo.TimeCount  = CamInfo.Time - CamInfo.TimeLast;
                    CamInfo.TimeLast   = CamInfo.Time;
                    emit showFPS();
                }

                if(ShowImgMutex.tryLock())
                {
                    memcpy(CamInfo.ShowImgData, CamInfo.ImgData, CamInfo.ImgW*CamInfo.ImgH*CamInfo.ImgB*CamInfo.ImgC/8);
                    CamInfo.ShowImgW = CamInfo.ImgW;
                    CamInfo.ShowImgH = CamInfo.ImgH;
                    CamInfo.ShowImgB = CamInfo.ImgB;
                    CamInfo.ShowImgC = CamInfo.ImgC;
                    emit gotImageData();
                    ShowImgMutex.unlock();
                }
            }
        }
        ret = QHYCCD_ERROR;

        if(CamInfo.Quit)
            break;
    }

    CamInfo.Quit = false;
}
